/*
  binforge - Repository system for Windows
  Copyright (C) 2009 Younes Serraj  Tous droits rservs.

  Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
  modifier suivant les termes de la GNU General Public License telle que
  publie par la Free Software Foundation : soit la version 3 de cette
  licence, soit ( votre gr) toute version ultrieure.

  Ce programme est distribu dans lespoir quil vous sera utile, mais SANS
  AUCUNE GARANTIE : sans mme la garantie implicite de COMMERCIALISABILIT
  ni dADQUATION  UN OBJECTIF PARTICULIER. Consultez la Licence Gnrale
  Publique GNU pour plus de dtails.

  Vous devriez avoir reu une copie de la Licence Gnrale Publique GNU avec
  ce programme ; si ce nest pas le cas, consultez :
  <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>

#include "binforge.h"

int init();
void destroy();
void readln(char *command, int maxlength);

int main() {
	package p;
	char command[2048];
	char buffer[2048];
	char choice;
	
	if (init() == false)
		return 0;
	
	while (1) {
		printf("\n$ ");
		readln(command, sizeof(command));
		
		if (!strcmp(command, "help")) {
			printf(" add name v.v.v.v filepath\n");
			printf(" update id name filename v.v.v.v\n");
			printf(" exists id\n");
			printf(" print id\n");
			printf(" extract id destination_directory\n");
			printf(" list\n");
			printf(" help\n");
			printf(" exit\n");
		}
		else if (!strcmp(command, "exit")) {
			break;
		}
		else if (!strcmp(left(command, 4), "add ")) {
			/*
			add name v.v.v.v filepath
			*/
			if (findchar(command, ' ', 2) == -1) {
				printf("Too few arguments.\n");
				continue;
			}
			sscanf(command, "add %s %d.%d.%d.%d", p.name, &p.version[0],
				&p.version[1], &p.version[2], &p.version[3]);
			strcpy(buffer,
				substr(command, findchar(command, ' ', 2)+1,
				strlen(command)-findchar(command, ' ', 2)+1
			));
			printf("Description: ");
			fflush(stdin);
			gets(p.description);
			pkg_make(&p, p.name, extract_filename(buffer), p.description,
				p.version[0], p.version[1], p.version[2], p.version[3]);
			if (pkg_add(&p, buffer))
				printf("Package added.\n");
			else {
				printf("An error occured during the operation.\n");
				printf("Make sure you wrote the command correctly and the source file exists.\n");
			}
		}
		else if (!strcmp(left(command, 7), "update ")) {
			/*
			update id name filename v.v.v.v
			*/
			sscanf(command, "update %d %s %s %d.%d.%d.%d", &p.id, p.name, buffer,
				&p.version[0], &p.version[1], &p.version[2],
				&p.version[3]);
			printf("Description:\n");
			fflush(stdin);
			gets(p.description);
			pkg_make(&p, p.name, buffer, p.description,
				p.version[0], p.version[1], p.version[2], p.version[3]);
			if (pkg_update(&p))
				printf("Package updated.\n");
			else {
				printf("An error occured during the operation.\n");
				printf("Make sure the package id exists.\n");
			}
		}
		else if (!strcmp(left(command, 7), "exists ")) {
			/*
			exists id
			*/
			sscanf(command, "exists %d", &p.id);
			if (pkg_exists(p.id) == false)
				printf("No.\n");
			else
				printf("Yes.\n");
		}
		else if (!strcmp(left(command, 6), "print ")) {
			/*
			print id
			*/
			sscanf(command, "print %d", &p.id);
			if (pkg_exists(p.id) == false)
				printf("No package associated to this id.\n");
			else {
				pkg_print(pkgdb_get(p.id));
			}
		}
		else if (!strcmp(left(command, 8), "extract ")) {
			/*
			extract id destdir
			*/
			if (findchar(command, ' ', 1) == -1) {
				printf("Too few arguments.\n");
				continue;
			}
			sscanf(command, "extract %d", &(p.id));
			strcpy(buffer,
				substr(command, findchar(command, ' ', 1)+1,
				strlen(command)-findchar(command, ' ', 1)+1
			));
			if (pkg_extract(p.id, buffer))
				printf("Package extracted.\n");
			else {
				printf("An error occured during the operation.\n");
				printf("Make sure the package id exists as well as the destination directory.\n");
			}
		}
		else if (!strcmp(command, "list")) {
			pkgdb_list();
		}
		else if (!strcmp(command, "clear")) {
			printf("Warning: this is an irreversible action!\n");
			printf("Are you sure you want to do it (y/n)? ");
			scanf(" %c", &choice);
			if (choice == 'y')
				pkg_clear();
		}
		else {
			printf("Unknown command.\n");
		}
	}
	
	destroy();
	printf("\nGood bye :)\n");
	return 0;
}

int init() {
	printf("binforge v%s\n", BINFORGE_VERSION);
	
	// Repository initialization
	printf("Repository initialization... ");
	if (pkgr_init() == false) {
		printf("failed.\nCould not initialize the repository.\n");
		return false;
	}
	else
		printf("done.\n");
	
	// Repository database opening
	printf("Repository database opening... ");
	if (pkgdb_open() == false) {
		printf("failed.\nCould not open the repository database.\n");
		return false;
	}
	else
		printf("done.\n");
	
	return true;
}

void destroy() {
	// Repository database closing
	printf("Repository database closing... ");
	pkgdb_close();
	printf("done.\n");
}

void readln(char *command, int maxlength) {
	fflush(stdin);
	fgets(command, maxlength, stdin);
	command[strlen(command)-1] = '\0';
}

